"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const rxjs_1 = require("rxjs");
const transform_1 = require("ng-packagr/lib/brocc/transform");
const nodes_1 = require("ng-packagr/lib/ng-v5/nodes");
const analyse_sources_di_1 = require("ng-packagr/lib/ng-v5/init/analyse-sources.di");
const entry_point_di_1 = require("ng-packagr/lib/ng-v5/entry-point.di");
const compile_ngc_di_1 = require("ng-packagr/lib/ng-v5/entry-point/ts/compile-ngc.di");
const write_bundles_di_1 = require("ng-packagr/lib/ng-v5/entry-point/write-bundles.di");
const write_package_di_1 = require("ng-packagr/lib/ng-v5/entry-point/write-package.di");
const HOOK_HANDLERS = ['analyseSources'];
const TRANSFORM_PROVIDER_MAP = {
    analyseSources: analyse_sources_di_1.ANALYSE_SOURCES_TRANSFORM,
    entryPoint: entry_point_di_1.ENTRY_POINT_TRANSFORM,
    compileNgc: compile_ngc_di_1.COMPILE_NGC_TRANSFORM,
    writeBundles: write_bundles_di_1.WRITE_BUNDLES_TRANSFORM,
    writePackage: write_package_di_1.WRITE_PACKAGE_TRANSFORM
};
function createHookProviders(hooksConfig) {
    const providers = [];
    const hookNames = Object.keys(TRANSFORM_PROVIDER_MAP);
    for (const key of hookNames) {
        if (hooksConfig[key]) {
            providers.push(createHookProvider(key, hooksConfig[key]));
        }
    }
    return providers;
}
exports.createHookProviders = createHookProviders;
function createHookProvider(sourceHookName, hookConfig) {
    const originalProvider = TRANSFORM_PROVIDER_MAP[sourceHookName];
    if (!originalProvider) {
        throw new Error(`Invalid source hook name, ${sourceHookName} is not a recognized hook`);
    }
    const clonedProvider = Object.assign({}, originalProvider);
    clonedProvider.useFactory = (...args) => {
        const { before, replace, after } = hookConfig;
        const runners = [];
        const isEntryPointHandler = HOOK_HANDLERS.indexOf(sourceHookName) === -1;
        if (before) {
            runners.push(createHookTransform(before, isEntryPointHandler));
        }
        runners.push(replace ? createHookTransform(replace, isEntryPointHandler) : originalProvider.useFactory(...args));
        if (after) {
            runners.push(createHookTransform(after, isEntryPointHandler));
        }
        return rxjs_1.pipe(...runners);
    };
    return clonedProvider;
}
exports.createHookProvider = createHookProvider;
function createHookTransform(handler, isEntryPointHandler) {
    return transform_1.transformFromPromise((graph) => __awaiter(this, void 0, void 0, function* () {
        if (isEntryPointHandler) {
            const fn = handler;
            const entryPointNode = graph.find(nodes_1.isEntryPointInProgress());
            return Promise.resolve(fn(entryPointNode, graph)).then(g => g || graph);
        }
        else {
            const fn = handler;
            return Promise.resolve(fn(graph)).then(g => g || graph);
        }
    }));
}
//# sourceMappingURL=create-hook-provider.js.map