"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const FS = require("fs");
const rxjs_1 = require("rxjs");
const operators_1 = require("rxjs/operators");
const devKitCore = require("@angular-devkit/core");
const build_ng_packagr_1 = require("@angular-devkit/build-ng-packagr");
const ngPackagr = require("ng-packagr");
// TODO: Remove when issue is fixed.
require("./workaround-issue-1189");
const create_hook_provider_1 = require("./create-hook-provider");
const DEFAULT_TSCONFIG_OPTIONS = {
    moduleResolution: 'node',
    module: 'commonjs',
    target: 'es6',
    lib: [
        'es2017',
        'dom'
    ],
};
// Instead of re-writing NgPackagrBuilder in full, just for adding some providers, we will patch
// the NgPackager instead... ohh my!
class NgPackagrBuilder extends build_ng_packagr_1.NgPackagrBuilder {
    run(builderConfig) {
        this.options = builderConfig.options;
        const { build, watch } = ngPackagr.NgPackagr.prototype;
        return rxjs_1.from(this.getTransformerHooks(this.options.transformConfig))
            .pipe(operators_1.tap(transformerHooks => {
            const providers = create_hook_provider_1.createHookProviders(transformerHooks);
            ngPackagr.NgPackagr.prototype.build = function () {
                this.withProviders(providers);
                return build.call(this);
            };
            ngPackagr.NgPackagr.prototype.watch = function () {
                this.withProviders(providers);
                return watch.call(this);
            };
        }), operators_1.switchMap(() => super.run(builderConfig)), operators_1.tap(buildEvent => {
            ngPackagr.NgPackagr.prototype.build = build;
            ngPackagr.NgPackagr.prototype.watch = watch;
        }));
    }
    getTransformerHooks(transformerPath) {
        const root = this.context.workspace.root;
        const tPath = devKitCore.resolve(root, devKitCore.normalize(transformerPath));
        if (FS.existsSync(tPath)) {
            if (/\.ts$/.test(tPath) && !require.extensions['.ts']) {
                const tsNodeOptions = {};
                if (this.options.transformTsConfig) {
                    tsNodeOptions.project = this.options.transformTsConfig;
                }
                else {
                    tsNodeOptions.compilerOptions = DEFAULT_TSCONFIG_OPTIONS;
                }
                require('ts-node').register(tsNodeOptions);
            }
            const transformHooks = require(tPath);
            if (typeof transformHooks === 'function') {
                const ctx = {
                    logger: this.context.logger,
                    root,
                };
                return Promise.resolve(transformHooks(ctx));
            }
            else {
                return Promise.resolve(transformHooks);
            }
        }
        ;
        return Promise.resolve({});
    }
}
exports.NgPackagrBuilder = NgPackagrBuilder;
exports.default = NgPackagrBuilder;
//# sourceMappingURL=index.js.map